@@ -7,8 +7,14 @@
 #define MVM_NFA_EDGE_CHARCLASS_NEG     5
 #define MVM_NFA_EDGE_CHARLIST          6
 #define MVM_NFA_EDGE_CHARLIST_NEG      7
+#define MVM_NFA_EDGE_SUBRULE           8
 #define MVM_NFA_EDGE_CODEPOINT_I       9
 #define MVM_NFA_EDGE_CODEPOINT_I_NEG   10
+#define MVM_NFA_EDGE_GENERIC_VAR       11
+#define MVM_NFA_EDGE_CHARRANGE         12
+#define MVM_NFA_EDGE_CHARRANGE_NEG     13
+#define MVM_NFA_EDGE_CODEPOINT_LL      14
+#define MVM_NFA_EDGE_CODEPOINT_I_LL    15
 
 /* State entry. */
 struct MVMNFAStateInfo {
@@ -18,8 +24,8 @@ struct MVMNFAStateInfo {
         MVMint64   i;
         MVMString *s;
         struct {
-            MVMCodepoint32 uc;
-            MVMCodepoint32 lc;
+            MVMGrapheme32 uc;
+            MVMGrapheme32 lc;
         } uclc;
     } arg;
 };
