@@ -23,6 +23,17 @@ struct MVMSpeshFacts {
         MVMnum64 n64;
         MVMString *s;
     } value;
+
+    /* The instruction that writes the register (noting we're in SSA form, so
+     * this is unique). */
+    MVMSpeshIns *writer;
+
+    /* The deoptimization index in effect at the point of declaration, or -1
+     * if none yet. */
+    MVMint32 deopt_idx;
+
+    /* The log guard the facts depend on, if any. */
+    MVMuint32 log_guard;
 };
 
 /* Various fact flags. */
@@ -34,6 +45,11 @@ struct MVMSpeshFacts {
 #define MVM_SPESH_FACT_KNOWN_DECONT_TYPE    32  /* Has a known type after decont. */
 #define MVM_SPESH_FACT_DECONT_CONCRETE      64  /* Is concrete after decont. */
 #define MVM_SPESH_FACT_DECONT_TYPEOBJ       128 /* Is a type object after decont. */
+#define MVM_SPESH_FACT_FROM_LOG_GUARD       256 /* Depends on a guard being met. */
+#define MVM_SPESH_FACT_HASH_ITER            512 /* Is an iter over hashes. */
+#define MVM_SPESH_FACT_ARRAY_ITER           1024 /* Is an iter over arrays
+                                                    (mutually exclusive with HASH_ITER, but neither of them is necessarily set) */
+#define MVM_SPESH_FACT_KNOWN_BOX_SRC        2048 /* We know what register this value was boxed from */
 
 /* Discovers spesh facts and builds up information about them. */
 void MVM_spesh_facts_discover(MVMThreadContext *tc, MVMSpeshGraph *g);
