@@ -4,6 +4,21 @@ struct MVMSpeshGraph {
     /* The static frame this is the spesh graph for. */
     MVMStaticFrame *sf;
 
+    /* The callsite this spesh graph has been tailored to. */
+    MVMCallsite *cs;
+
+    /* The bytecode we're building the graph out of. */
+    MVMuint8 *bytecode;
+
+    /* Exception handler map for that bytecode. */
+    MVMFrameHandler *handlers;
+
+    /* The size of the bytecode we're building the graph out of. */
+    MVMuint32 bytecode_size;
+
+    /* Number of exception handlers. */
+    MVMuint32 num_handlers;
+
     /* The entry basic block. */
     MVMSpeshBB *entry;
 
@@ -15,10 +30,16 @@ struct MVMSpeshGraph {
     MVMuint16 *fact_counts;
 
     /* Argument guards added. */
-    MVMSpeshGuard *guards;
+    MVMSpeshGuard *arg_guards;
 
-    /* Number of guards we have. */
-    MVMint32 num_guards;
+    /* Number of argument guards we have. */
+    MVMint32 num_arg_guards;
+
+    /* Log-based guards added. */
+    MVMSpeshLogGuard *log_guards;
+
+    /* Number of log-based guards we have. */
+    MVMint32 num_log_guards;
 
     /* Memory blocks we allocate to store spesh nodes, and which we free along
      * with the graph. Contains a link to previous blocks. */
@@ -40,12 +61,38 @@ struct MVMSpeshGraph {
     MVMint32  num_deopt_addrs;
     MVMint32  alloc_deopt_addrs;
 
+    /* Table of information about inlines, laid out in order of nesting
+     * depth. Thus, going through the table in order and finding when we
+     * are within the bounds will show up each call frame that needs to
+     * be created in deopt. */
+    MVMSpeshInline *inlines;
+    MVMint32 num_inlines;
+
     /* Logging slots, along with the number of them. */
-    MVMCollectable **log_slots;
     MVMint32 num_log_slots;
+    MVMCollectable **log_slots;
 
     /* Number of basic blocks we have. */
     MVMint32 num_bbs;
+
+    /* The list of local types (only set up if we do inlines). */
+    MVMuint16 *local_types;
+
+    /* The list of lexical types (only set up if we do inlines). */
+    MVMuint16 *lexical_types;
+
+    /* The total number of locals, accounting for any inlining done and
+     * added temporaries. */
+    MVMuint16 num_locals;
+
+    /* The total number of lexicals, accounting for any inlining done. */
+    MVMuint16 num_lexicals;
+
+    /* Temporary local registers added to aid transformations, along with a
+     * count of the number we have and have allocated space for so far. */
+    MVMuint16          num_temps;
+    MVMuint16          alloc_temps;
+    MVMSpeshTemporary *temps;
 };
 
 /* The default allocation chunk size for memory blocks used to store spesh
@@ -67,6 +114,19 @@ struct MVMSpeshMemBlock {
     MVMSpeshMemBlock *prev;
 };
 
+/* A temporary register, added to support transformations. */
+struct MVMSpeshTemporary {
+    /* The number of the local along with the current SSA index. */
+    MVMuint16 orig;
+    MVMuint16 i;
+
+    /* What kind of register is it? */
+    MVMuint16 kind;
+
+    /* Is it currently in use? */
+    MVMuint16 in_use;
+};
+
 /* A basic block in the graph (sequences of instructions where control will
  * always enter at the start and leave at the end). */
 struct MVMSpeshBB {
@@ -98,6 +158,9 @@ struct MVMSpeshBB {
     /* Index (just an ascending integer along the linear_next chain), used as
      * the block identifier in dominance computation and for debug output. */
     MVMint32 idx;
+
+    /* Is this block an inlining of another one? */
+    MVMint32 inlined;
 };
 
 /* The SSA phi instruction. */
@@ -106,7 +169,7 @@ struct MVMSpeshBB {
 /* An instruction in the spesh graph. */
 struct MVMSpeshIns {
     /* Instruction information. */
-    MVMOpInfo *info;
+    const MVMOpInfo *info;
 
     /* Operand information. */
     MVMSpeshOperand *operands;
@@ -138,8 +201,8 @@ union MVMSpeshOperand {
         MVMuint16 outers;
     } lex;
     struct {
-        MVMuint16 orig; /* Original register number. */
         MVMint32  i;    /* SSA-computed version. */
+        MVMuint16 orig; /* Original register number. */
     } reg;
 };
 
@@ -155,6 +218,7 @@ struct MVMSpeshAnn {
     union {
         MVMint32 frame_handler_index;
         MVMint32 deopt_idx;
+        MVMint32 inline_idx;
     } data;
 };
 
@@ -164,9 +228,16 @@ struct MVMSpeshAnn {
 #define MVM_SPESH_ANN_FH_GOTO       3
 #define MVM_SPESH_ANN_DEOPT_ONE_INS 4
 #define MVM_SPESH_ANN_DEOPT_ALL_INS 5
+#define MVM_SPESH_ANN_INLINE_START  6
+#define MVM_SPESH_ANN_INLINE_END    7
+#define MVM_SPESH_ANN_DEOPT_INLINE  8
+#define MVM_SPESH_ANN_DEOPT_OSR     9
 
 /* Functions to create/destory the spesh graph. */
-MVMSpeshGraph * MVM_spesh_graph_create(MVMThreadContext *tc, MVMStaticFrame *sf);
+MVMSpeshGraph * MVM_spesh_graph_create(MVMThreadContext *tc, MVMStaticFrame *sf, MVMuint32 cfg_only);
+MVMSpeshGraph * MVM_spesh_graph_create_from_cand(MVMThreadContext *tc, MVMStaticFrame *sf,
+    MVMSpeshCandidate *cand, MVMuint32 cfg_only);
+MVMSpeshBB * MVM_spesh_graph_linear_prev(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshBB *search);
 void MVM_spesh_graph_mark(MVMThreadContext *tc, MVMSpeshGraph *g, MVMGCWorklist *worklist);
 void MVM_spesh_graph_destroy(MVMThreadContext *tc, MVMSpeshGraph *g);
-void * MVM_spesh_alloc(MVMThreadContext *tc, MVMSpeshGraph *g, size_t bytes);
+MVM_PUBLIC void * MVM_spesh_alloc(MVMThreadContext *tc, MVMSpeshGraph *g, size_t bytes);
